/* $Id: abbotout2raw.c,v 1.4 1998/03/18 02:58:32 dpwe Exp $

   $Log: abbotout2raw.c,v $
   Revision 1.4  1998/03/18 02:58:32  dpwe
   byteswap support

   Revision 1.3  1998/02/18 00:50:19  dpwe
    - added ascii=1 option for ascii (instead of binary) output

   Revision 1.2  1997/09/10 01:21:34  dpwe
   - error for input file not to end with an end-marked frame
   - reports total number of utterances seen (for yucks)

   Revision 1.1  1997/09/10 00:30:45  dpwe
   Initial revision

   Revision 1.3  1997/06/28 00:43:29  dpwe
   fixed argc=1 case

   Revision 1.2  1996/11/06 00:49:03  bedk
   Changed --- abbot output format is also the ICSI online feature format.

   Revision 1.1  1996/11/05 00:43:39  bedk
   Initial revision

   Program to conver the abbot-format online output back into the 
   raw binary feature files so typical of rasta.  Used to test 
   the Abbot I/O option for rasta. */

#include <stdio.h>
#include <stdlib.h>
#include <assert.h>
#include <string.h>

#include "byteswap.h"

/* split command-line arguments of the form arg=val
   into arg and val */ 

void
getargval(char* s, char** arg, char** val)
{
  char* eq;

  eq = strchr(s, '=');
  assert(eq != NULL);

  *arg = malloc((eq - s + 1) * sizeof(char));
  strncpy(*arg, s, eq - s);
  *(*arg + (eq - s)) = '\000';

  *val = malloc((strlen(s) - (eq - s)));
  strncpy(*val, eq + 1, strlen(s) - (eq - s) - 1);
  *(*val + strlen(s) - (eq - s) - 1) = '\000';
}

/* return next item in a comma-separated list */

char* list = NULL;
char*
getnextitem(char* s)
{
  static char* p = NULL;
  char* next = NULL;
  unsigned int l;

  if (list == NULL)
    /* Initialization */
    {
      list = strdup(s);
      p = list;
      return NULL;
    }
  else if (*p == '\000')
    {
      if (s != NULL) free(s);
      free(list);
      list = NULL;
      p = NULL;
      return next;
    }
  else
    {
      if (s != NULL) free(s);
      l = strcspn(p, ",");
      next = malloc((l + 1) * sizeof(char));
      strncpy(next, p, l);
      *(next + l) = '\000';
      p += l;
      if (*p == ',') p++;
      return next;
    }
}

int
main(int argc, char** argv)
{
  char** argv_endp = argv + argc;
  char** argp;
  char* infile = NULL;
  int framesize = 9;
  int asciiout  = 0;
  unsigned char eos;
  float* b1;
  FILE* fp;
  int done;
  int nutts = 0;

  /* parse arguments (and handle argc=1 case!) */

  for (argp = argv + 1; argp < argv_endp; argp++)
    {
      char* arg;
      char* val;
      getargval(*argp, &arg, &val);

      if (strcmp(arg, "framesize") == 0)
        {
          sscanf(val, "%d", &framesize);
        }
      else if (strcmp(arg, "infiles") == 0)
        {
          /* initialize file name iterator */
          getnextitem(val);
        }
      else if (strcmp(arg, "ascii") == 0)
        {
          /* ascii=1 -> write ascii, not binary, output */
	  sscanf(val, "%d", &asciiout);
        }
      else
        {
          fprintf(stderr, "Invalid argument %s\n", arg);
          exit(1);
        }
      free(arg);
      free(val);
    }

  /* did we get initialized? (weird way to tell!) */
  if(list == NULL) {
      fprintf(stderr, "abbotout2raw: you must specify infiles=...\n");
      exit(-1);
  }
  if(framesize <= 0) {
      fprintf(stderr, "abbotout2raw: you must specify framesize=...\n");
      exit(-1);
  }

  b1 = (float*) malloc(framesize * sizeof(float));

  /* loop over input files */
  while ((infile = getnextitem(infile)) != NULL)
    {
      if ((fp = fopen(infile, "r")) == NULL)
        {
          fprintf(stderr, "Unable to open %s", infile);
          exit(1);
        }

      done = 0;
      eos = 0;
      fread((char*) &eos, sizeof(char), 1, fp);
      while (!done \
	     && fread_swap((char*) b1, sizeof(float), framesize, fp, 0) == framesize)
        {
	  if(asciiout) {
	      /* write a frame of ascii values */
	      int i;
	      for (i = 0; i < framesize; ++i) {		  
		  if(i) 	fprintf(stdout, " ");
		  fprintf(stdout, "%g", b1[i]);
		  /* %g is what rasta's ascout uses, so copy it here */
	      }
	      fprintf(stdout, "\n");		  
	  } else {
	      fwrite_swap((char*) b1, sizeof(float), framesize, stdout, 0);
	  }
	  if(eos != 0 && eos != 0x80) {
	      fprintf(stderr, "abbotout2raw: eos of 0x%x unexpected!\n", 
		      eos);
	      exit(-1);
	  }
	  if(eos) {
	      ++nutts;
	      /* done = 1; */
	      /* no, keep copying until EOF (for multi-utterance tests) */
	  }
	  if (!done) {	  
	      done = (fread((char*) &eos, sizeof(char), 1, fp) == 0);
	  }
	}

      fclose(fp);
    }

  if(eos != 0x80) {
      fprintf(stderr, "%s: file does not end with a 'final frame'\n", 
	      argv[0]);
      exit(-1);
  }

  fprintf(stderr, "%s: total num utts=%d\n", argv[0], nutts);

  free(b1);
  return 0;
}
