/* 
 * byteswap.h
 * 
 * define versions of fread and fwrite that automatically byteswap
 * 1998mar17 dpwe@icsi.berkeley.edu
 * $Header: /u/drspeech/src/rasta/testsuite/RCS/byteswap.h,v 1.1 1998/03/18 02:58:32 dpwe Exp $
 */

#include <assert.h>

static void swapb(void *buf, int size, int nitems) {
  /* swap pairs or quads of bytes in a buffer */
  if(size==2) {
    short *sbuf = buf;
    int x;
    assert(sizeof(short)==2);
    while(nitems--) {
      x = *sbuf;
      *sbuf++ = ((x & 0xFF) << 8) + ((x >> 8) & 0xFF);
    }
  } else if(size==4) {
    int *ibuf = buf;
    int x;
    assert(sizeof(int)==4);
    while(nitems--) {
      x = *ibuf;
      *ibuf++ = ((x & 0xFFL) << 24L) + ((x & 0xFF00L) << 8L) \
                + ((x & 0xFF0000L) >> 8L) + ((x >> 24L) & 0xFF);
    }
  }
  /* ignore other sizes */
}

static int fread_swap(void *buf, int size, int nitems, FILE *stream, int swap) {
  /* Do an fread of a certain sized item, swap if littlendian 
     or swap explicitly requested (but not both) */
  int i=1;
  char *pc = (char *)&i;
  int littleendian = (*pc)?1:0;   /* first byte of 0x0001 is not zero */
  int wantswap     = (swap)?1:0;

  int nread = fread(buf, size, nitems, stream);
  if (wantswap != littleendian) {
      swapb(buf, size, nread);
  }
  return nread;
}

static int fwrite_swap(void *buf, int size, int nitems, FILE *stream, int swap) {
  /* Do an fwrite of items, swap bytes if machine is little-endian, 
     also if swap is true (but not if both!) */
  int i=1;
  char *pc = (char *)&i;
  int littleendian = (*pc)?1:0;   /* first byte of 0x0001 is not zero */
  int wantswap     = (swap)?1:0;
  int nwrit;

  if(wantswap != littleendian) {
    swapb(buf, size, nitems);
  }
  nwrit = fwrite(buf, size, nitems, stream);
  /* swap back */
  if(wantswap != littleendian) {
    swapb(buf, size, nitems);
  }
  return nwrit;
}

