/* $Id: raw2abbotin.c,v 1.3 1998/03/18 02:58:32 dpwe Exp $

   $Log: raw2abbotin.c,v $
   Revision 1.3  1998/03/18 02:58:32  dpwe
   byteswap support

   Revision 1.2  1997/09/06 02:02:47  dpwe
   handle argc=1 without crashing

   Revision 1.1  1996/10/18 23:15:16  bedk
   Initial revision


   Program to take a list of raw binary speech data, merge it
   together, and write it to stdout in Abbot wav format.  Used to test
   the Abbot I/O option for rasta.  */

#include <stdio.h>
#include <stdlib.h>
#include <assert.h>
#include <string.h>

#include "byteswap.h"

/* split command-line arguments of the form arg=val
   into arg and val */ 

void
getargval(char* s, char** arg, char** val)
{
  char* eq;

  eq = strchr(s, '=');
  assert(eq != NULL);

  *arg = malloc((eq - s + 1) * sizeof(char));
  strncpy(*arg, s, eq - s);
  *(*arg + (eq - s)) = '\000';

  *val = malloc((strlen(s) - (eq - s)));
  strncpy(*val, eq + 1, strlen(s) - (eq - s) - 1);
  *(*val + strlen(s) - (eq - s) - 1) = '\000';
}

/* return next item in a comma-separated list */

char*
getnextitem(char* s)
{
  static char* list = NULL;
  static char* p = NULL;
  char* next = NULL;
  unsigned int l;

  if (list == NULL)
    /* Initialization */
    {
      list = strdup(s);
      p = list;
      return NULL;
    }
  else if (*p == '\000')
    {
      if (s != NULL) free(s);
      free(list);
      list = NULL;
      p = NULL;
      return next;
    }
  else
    {
      if (s != NULL) free(s);
      l = strcspn(p, ",");
      next = malloc((l + 1) * sizeof(char));
      strncpy(next, p, l);
      *(next + l) = '\000';
      p += l;
      if (*p == ',') p++;
      return next;
    }
}

int
main(int argc, char** argv)
{
  unsigned short b;
  char** argv_endp = argv + argc;
  char** argp;
  char* infile = NULL;
  unsigned int bigendian = 1;
  FILE* fp;

  /* parse arguments (and hanlde argc=1 case!) */

  for (argp = argv + 1; argp < argv_endp; argp++)
    {
      char* arg;
      char* val;
      getargval(*argp, &arg, &val);

      if (strcmp(arg, "byteorder") == 0)
        {
          if (strcmp(val, "big") == 0)
            {
              bigendian = 1;
            }
          else if (strcmp(val, "little") == 0)
            {
              bigendian = 0;
            }
          else
            {
              fprintf(stderr, "Invalid value for byteorder.\n");
              exit(1);
            }
        }
      else if (strcmp(arg, "infiles") == 0)
        {
          /* initialize file name iterator */
          getnextitem(val);
        }
      else
        {
          fprintf(stderr, "Invalid argument %s\n", arg);
          exit(1);
        }
      free(arg);
      free(val);
    }

  /* loop over input files */
  while ((infile = getnextitem(infile)) != NULL)
    {
      if ((fp = fopen(infile, "r")) == NULL)
        {
          fprintf(stderr, "Unable to open %s", infile);
          exit(1);
        }
      while (fread_swap((char*) &b, sizeof(short), 1, fp, 0) == 1)
        {
          if (b == 0x8000) b = 0x8001;
	  fwrite_swap((char*) &b, sizeof(short), 1, stdout, !bigendian);
	}
      b = 0x8000;
      fwrite_swap((char*) &b, sizeof(short), 1, stdout, !bigendian);

      fclose(fp);
    }

  return 0;
}
