/* $Id: raw2abbotout.c,v 1.4 1998/03/18 02:58:32 dpwe Exp $

   $Log: raw2abbotout.c,v $
   Revision 1.4  1998/03/18 02:58:32  dpwe
   byteswap support

   Revision 1.3  1997/06/28 00:43:29  dpwe
   fixed argc=1 case

   Revision 1.2  1996/11/06 00:49:03  bedk
   Changed --- abbot output format is also the ICSI online feature format.

   Revision 1.1  1996/11/05 00:43:39  bedk
   Initial revision


   Program to take a list of raw binary rasta output files, merge them
   together, and write them to stdout in Abbot format.  Used to test
   the Abbot I/O option for rasta.  */

#include <stdio.h>
#include <stdlib.h>
#include <assert.h>
#include <string.h>

#include "byteswap.h"

/* split command-line arguments of the form arg=val
   into arg and val */ 

void
getargval(char* s, char** arg, char** val)
{
  char* eq;

  eq = strchr(s, '=');
  assert(eq != NULL);

  *arg = malloc((eq - s + 1) * sizeof(char));
  strncpy(*arg, s, eq - s);
  *(*arg + (eq - s)) = '\000';

  *val = malloc((strlen(s) - (eq - s)));
  strncpy(*val, eq + 1, strlen(s) - (eq - s) - 1);
  *(*val + strlen(s) - (eq - s) - 1) = '\000';
}

/* return next item in a comma-separated list */

char*
getnextitem(char* s)
{
  static char* list = NULL;
  static char* p = NULL;
  char* next = NULL;
  unsigned int l;

  if (list == NULL)
    /* Initialization */
    {
      list = strdup(s);
      p = list;
      return NULL;
    }
  else if (*p == '\000')
    {
      if (s != NULL) free(s);
      free(list);
      list = NULL;
      p = NULL;
      return next;
    }
  else
    {
      if (s != NULL) free(s);
      l = strcspn(p, ",");
      next = malloc((l + 1) * sizeof(char));
      strncpy(next, p, l);
      *(next + l) = '\000';
      p += l;
      if (*p == ',') p++;
      return next;
    }
}

int
main(int argc, char** argv)
{
  char** argv_endp = argv + argc;
  char** argp;
  char* infile = NULL;
  unsigned int bigendian = 1;
  int framesize = 9;
  unsigned char eos;
  float* b1;
  float* b2;
  FILE* fp;

  /* parse arguments (and handle argc=1 case!) */

  for (argp = argv + 1; argp < argv_endp; argp++)
    {
      char* arg;
      char* val;
      getargval(*argp, &arg, &val);

      if (strcmp(arg, "byteorder") == 0)
        {
          if (strcmp(val, "big") == 0)
            {
              bigendian = 1;
            }
          else if (strcmp(val, "little") == 0)
            {
              bigendian = 0;
            }
          else
            {
              fprintf(stderr, "Invalid value for byteorder.\n");
              exit(1);
            }
        }
      else if (strcmp(arg, "framesize") == 0)
        {
          sscanf(val, "%d", &framesize);
        }
      else if (strcmp(arg, "infiles") == 0)
        {
          /* initialize file name iterator */
          getnextitem(val);
        }
      else
        {
          fprintf(stderr, "Invalid argument %s\n", arg);
          exit(1);
        }
      free(arg);
      free(val);
    }

  b1 = (float*) malloc(framesize * sizeof(float));
  b2 = (float*) malloc(framesize * sizeof(float));

  /* loop over input files */
  while ((infile = getnextitem(infile)) != NULL)
    {
      if ((fp = fopen(infile, "r")) == NULL)
        {
          fprintf(stderr, "Unable to open %s", infile);
          exit(1);
        }

      eos = 0;
      fread_swap((char*) b2, sizeof(float), framesize, fp, 0);
      while (fread_swap((char*) b1, sizeof(float), framesize, fp, 0) == framesize)
        {
          float* t = b1;
          b1 = b2;
          b2 = t;

          fwrite((char*) &eos, 1, 1, stdout);
          fwrite_swap((char*) b1, sizeof(float), framesize, stdout, 0);
        }

      eos = 0x80;
      fwrite((char*) &eos, 1, 1, stdout);
      fwrite_swap((char*) b2, sizeof(float), framesize, stdout, 0);

      fclose(fp);
    }

  free(b1);
  free(b2);
  return 0;
}
