const char* rcsid = "$Header: /u/drspeech/src/rasta/testsuite/RCS/raw2onlftr.c,v 1.2 1998/03/18 02:58:32 dpwe Exp $";

/*
  raw2onlftr.c: converts binary raw features to online features

  This program simply adds a single flag byte to the start of each
  frame.  This flag is normally 0x00, but has the value 0x80 for
  the last frame in the sentence.

  Author: Eric Fosler fosler@icsi

  $Id: raw2onlftr.c,v 1.2 1998/03/18 02:58:32 dpwe Exp $

  $Log: raw2onlftr.c,v $
  Revision 1.2  1998/03/18 02:58:32  dpwe
  byteswap support

  Revision 1.1  1997/06/26 23:42:55  dpwe
  Initial revision

  Revision 1.3  1996/03/11 22:42:21  davidj
  Added RCS header.

  Revision 1.2  1996/03/11 22:39:17  davidj
  Fixed some warnings.

  Revision 1.1  1996/01/09 22:37:20  fosler
  Initial revision

  */

#include <stdio.h>
#include <string.h>
#include <stdlib.h>
#include <ctype.h>
/* #include <icsiargs.h> */

#include "byteswap.h"

#define EOS (char)0x80
#define NOTEOS (char)0x00

int feat_size=4;
int num_feats=0;
char *progname;

/* icsiargtab_t argtab[]={
  {"size","Size in bytes of each feature",
   ARG_INT,&feat_size,ARG_OPT},
  {"width","Number of features per frame in raw file",
   ARG_INT,&num_feats,ARG_REQ},
  { 0 }
}; */

/* split command-line arguments of the form arg=val
   into arg and val */ 
int getargval(const char* s, char** arg, char** val)
{
  char* eq;

  eq = strchr(s, '=');
  if(eq == NULL) return 0;

  *arg = malloc((eq - s + 1) * sizeof(char));
  strncpy(*arg, s, eq - s);
  *(*arg + (eq - s)) = '\000';

  *val = malloc((strlen(s) - (eq - s)));
  strncpy(*val, eq + 1, strlen(s) - (eq - s) - 1);
  *(*val + strlen(s) - (eq - s) - 1) = '\000';
  return 1;
}

int main(int argc, const char **argv) {
  extern int feat_size;
  extern int num_feats;
  char *curfeats, *oldfeats, *temp;
  int num_bytes;
  int check;
  char eofflag;
  int valid=0;

  int i = 1;

/*  icsiargs(argtab,&argc,&argv,&progname); */

  while (i < argc) {
      char *arg, *val;
      if (getargval(argv[i], &arg, &val) == 0) { arg = ""; } /* force err */
      if(strcmp(arg, "size")==0) {
	  feat_size = atoi(val);
      } else if(strcmp(arg, "width")==0) {
	  num_feats = atoi(val);
      } else {
	  fprintf(stderr, "Unrecognized option: '%s'\n", argv[i]);
      }
      ++i;
  }
  if (num_feats <= 0) {
      fprintf(stderr, "Usage: %s width=<n> [size=4] < stdin > stdout\n", argv[0]);
      fprintf(stderr, "  width is the number of features per frame [REQUIRED],\n");
      fprintf(stderr, "  size is the number of bytes per feature.\n");
      exit(-1);
  }

  num_bytes=feat_size*num_feats;

  curfeats=(char *)malloc(num_bytes*sizeof(char));
  oldfeats=(char *)malloc(num_bytes*sizeof(char));

  while(!feof(stdin)) {
    check=fread_swap(curfeats, feat_size, num_feats, stdin, 0);
    if (check==0) { /* reached EOF */
      if (valid) {
	eofflag=EOS;
	fwrite(&eofflag,sizeof(char),1,stdout);
	fwrite_swap(oldfeats, feat_size, num_feats, stdout, 0);
      }
    }
    else if (check!=num_feats) {
      fprintf(stderr, 
	      "%s: Truncated frame seen (%d of %d bytes), exiting\n",
	      progname, check*feat_size, num_bytes*feat_size);
      exit(2);
    }
    else if (valid) {
      eofflag=NOTEOS;
      fwrite(&eofflag,sizeof(char),1,stdout);
      fwrite_swap(oldfeats, feat_size, num_feats, stdout, 0);
    }
    valid=1;
    temp=oldfeats;
    oldfeats=curfeats;
    curfeats=temp;
  }
  exit(0);
}
    

